<?php
/**
 * @file class.paukr.php
 * @copyright Sebastian Beyl, Zentrum fuer LehrerInnenbildung, Universitaet zu Koeln, 2020
 * @version 20201009
 *
 * PAUKR class wraps the PAUKR REST API. It is much more easier to use this class inside your
 * PHP project instead of using the PAUKR REST API directly.
 */


// Define the PAUKR STANDARD Server URL. It is possible to define an alternative server
// if you want pipe or proxy the paukr server.
if(!defined('PAUKR_SERVER_URL')) {
    define('PAUKR_SERVER_URL', 'paukr.zfl.uni-koeln.de');
}

/**
 * Class Paukr
 *
 * this class communicates with the PAUKR system over the REST API
 */
class Paukr {


    /**
     * the bearer token that has to be set in any PAUKR REST API as Authorization Header
     *
     * @var string
     */
    private $token;


    /**
     * complete server string to the API with protocol and server without ending slash
     *
     * @var string          PAUKR API url without ending slash
     */
    private $server;


    /**
     * Paukr constructor.
     *
     * If you want to communicate with the Paukr API, you have to build this
     * object to communicate. Before starting the communication, you have to set your token.
     * You can do this while constructing this object, or later
     * using the methods to set the token.
     *
     * @param string $token         [optional] PAUKR API token
     * @param string $protocol      [optional] https or http protocol, standard is https
     */
    public function __construct($token = '', $protocol = 'https') {

        // if server is set inside the construction, use it to set the server var
        $this->setServer( $protocol . '://' . PAUKR_SERVER_URL . '/api/v1');

        // if token is set inside the construction, use it to set the token var
        if (strlen($token) > 0) { $this->setToken($token); }

    }


    /**
     * returns all papers of one student, using its matrikel no. as search criteria. The
     * answer is an array of all paper names, that can be used for downloading it in the
     * next step.
     *
     * The matrikel no. has to be a seven digit string. If the matrikel no. is not valid
     * or is not registered inside the PAUKR system, the return answer of this method is
     * an empty array.
     *
     * @param int|string $matrikel         seven digit matrikel no. of the student
     * @return array                        available paper names
     */
    public function getPapers($matrikel) {

        // convert the matrikel no. to a string
        $matrikel = strval($matrikel);

        // check if the matrikel contains digits only
        if (!ctype_digit($matrikel)) { return array(); }

        // check if the matrikel is seven digits long
        if (strlen($matrikel) != 7) { return array(); }

        // build the request url
        $s_Url = '/matrikels/' . $matrikel . '/papers';

        // get json with papers key, decode it to an array
        $json_Papers = $this->get($s_Url);
        $arr_Papers = json_decode($json_Papers, true);

        // check if there is a papers key
        if (!isset($arr_Papers['papers'])) { return array(); }

        // return the papers array
        return $arr_Papers['papers'];

    }


    /**
     * returns the file content of a paper as a string. The string can contain binary datas
     * that can be streamed to the output directly.
     *
     * @param int|string $matrikel          seven digit matrikel no. of the student
     * @param string $paper                 paper file name inside the PAUKR server
     * @return false|string                 file content or false if not available
     */
    public function getPaperFile($matrikel, $paper) {

        // convert the matrikel no. to a string
        $matrikel = strval($matrikel);

        // check if the matrikel contains digits only
        if (!ctype_digit($matrikel)) { return false; }

        // check if the matrikel is seven digits long
        if (strlen($matrikel) != 7) { return false; }

        // check if the paper is given
        if (strlen($paper) < 1) { return false; }

        // build the request url
        $s_Url = '/matrikels/' . $matrikel . '/papers/' . urlencode($paper);

        // get json with papers key, decode it to an array
        $json_Paper = $this->get($s_Url);
        $arr_Paper = json_decode($json_Paper, true);

        // check if there is a file key
        if (!isset($arr_Paper['file'])) { return false; }

        // return the papers array
        return base64_decode($arr_Paper['file']);

    }


    /**
     * write a new file to the PAUKR system. There are a set of parameters that have to be set
     * to valid values. The PAUKR system will generate a standardized file name. You can not define
     * a filename by yourself.
     *
     * All parameters have to be inside valid values. Here are the valid paramters that can change
     * inside the next versions:
     *
     * file: binary file stream. You can easily read the stream using file_get_contents
     *
     * matrikel: 7 digit matrikel no. of the student
     *
     * date: 8 digits, format YYYYMMDD, inside the time frame from 19970101 and 20500101
     *
     * studgrad: MA for master, BA for Bachelor
     *
     * type: Type of the file, allowed is: Abschlussarbeit, DrittGutA, ErstGutA, Zeugnisantrag, ZweitGutA
     *
     * filetype: pdf
     *
     * @param $file
     * @param $matrikel
     * @param $date
     * @param $studgrad
     * @param $type
     * @param $filetype
     * @return false
     */
    public function writePaperFile($file, $matrikel, $date, $studgrad, $type, $filetype) {

        // convert the matrikel no. to a string
        $matrikel = strval($matrikel);

        // check if the matrikel contains digits only
        if (!ctype_digit($matrikel)) { return false; }

        // check if the matrikel is seven digits long
        if (strlen($matrikel) != 7) { return false; }

        // prepare an array with the given datas
        $arr_Params = array(
            'date' => $date,
            'studgrad' => $studgrad,
            'type' => $type,
            'filetype' => $filetype,
            'file' => base64_encode($file)
        );

        // build the request url
        $s_Url = '/matrikels/' . $matrikel . '/papers';

        // try to write it
        $json_Paper = $this->post($s_Url, $arr_Params);
        $arr_Paper = json_decode($json_Paper, true);

        // check if there is a status key in the answer and if it is "success"
        if (!is_array($arr_Paper)) { return false; }
        if (!isset($arr_Paper['status'])) { return false; }
        if ($arr_Paper['status'] != 'success') { return false; }

        // everything seems to worked
        return true;

    }


    /**
     * Set the PAUKR server URL. This address has to set with protocol and base url
     * but without ending slash.
     *
     * @param string $server            PAUKR API url
     */
    private function setServer($server) {

        // remove an ending slash, because if this is set, it will result in double
        // slashes while building the calling urls
        $server = rtrim($server, '/');

        // set this to the server var
        $this->server = $server;

    }


    /**
     * Set the PAUKR API token. If the token is not defined using the constructor, it can be
     * set later or can be changed.
     *
     * @param string $token
     */
    public function setToken($token) {

        // set the token to the token var
        $this->token = $token;

    }


    /**
     * push a GET request to the API, using the token as a bearer authorization.
     *
     * @param string $url               url to read
     * @param array|false $params       [optional] parameter array to append to the request
     * @return bool|string              reponse to the request, false if error
     */
    private function get($url, $params = false) {

        // if params is not set, set it to an empty array
        if ($params == false) { $params = array(); }

        // build the url with all parameters
        $s_Url = $this->server . $url;
        $s_QuerySign = '?';
        foreach($params as $key => $value) {
            $s_Url .= $s_QuerySign . urlencode($key) . '=' . urlencode($value);
            $s_QuerySign = '&';
        }

        // initiate curl to get
        $ch = curl_init($s_Url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLINFO_HEADER_OUT, true);

        // make header ready to authentification with Paukr token
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Authorization: Bearer ' . $this->token
        ));

        // Submit the POST request
        $result = curl_exec($ch);

        // Close cURL session handle
        curl_close($ch);

        // return the result
        return $result;

    }

    /**
     * push a POST request to the API, using the token as a bearer authorization.
     *
     * @param string $url               url to read
     * @param array|false $params       [optional] parameter array to append to the request
     * @return bool|string              reponse to the request, false if error
     */
    private function post($url, $params = false) {

        // if params is not set, set it to an empty array
        if ($params == false) { $params = array(); }

        // build the url
        $s_Url = $this->server . $url;

        // initiate curl to get
        $ch = curl_init($s_Url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-Type:application/json',
            'Authorization: Bearer ' . $this->token
        ));

        // build the parameter payload
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));

        // Submit the POST request
        $result = curl_exec($ch);

        // Close cURL session handle
        curl_close($ch);

        // return the result
        return $result;

    }


}


